@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2021 Antonio Diaz Diaz.
+   Copyright (C) 2013-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -316,8 +316,7 @@ extern "C" void * grouper( void * arg )
     if( Exclude::excluded( filename ) ) continue;	// skip excluded files
     struct stat st;
     if( lstat( filename, &st ) != 0 )	// filename from command line
-      { show_file_error( filename, "Can't stat input file", errno );
-        set_error_status( 1 ); }
+      { show_file_error( filename, cant_stat, errno ); set_error_status( 1 ); }
     else if( nftw( filename, add_member_lz, 16,
                    cl_opts.dereference ? 0 : FTW_PHYS ) != 0 )
       exit_fail_mt();			// write error or OOM
