@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2021 Antonio Diaz Diaz.
+   Copyright (C) 2013-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,12 +19,10 @@
 
 #include <algorithm>
 #include <cerrno>
-#include <climits>	// for tarlz.h
 #include <cstdio>
 #include <cstring>
 #include <string>
 #include <vector>
-#include <pthread.h>	// for tarlz.h
 #include <stdint.h>
 #include <unistd.h>
 
